/*
This file is part of "Avanor, the Land of Mystery" roguelike game
Home page: http://www.avanor.com/
Copyright (C) 2000-2003 Vadim Gaidukevich

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "location.h"
#include "other_misc.h"
#include "xapi.h"

//******* palletes script langugage ******* //
// Some simbols has default values, but can be overrided
// '.' - GREEN GRASS
// ',' - CAVE FLOOR
// ';' - STONE FLOOR
// '#' - STONE WALL
// '=' - WATER
// '&' - GREEN TREE
// 'X' - FENCE
// '+' - CLOSED DOOR
// '/' - OPENED DOOR
// '^' - TRAP DOOR


char * small_town_view[] = {
"..............................................",
".#####.#####.#####.#####.#####.#####.#####....",
".#111#.#111#.#111#.#111#.#111#.#111#.#111#....",
".##+##.##+##.##+##.##+##.##+##.##+##.##+##....",
"2222222222222222222222222222222222222222222222",
".##+##......22&=&22############2#####+######..",
".#111#......22=&=22#1111111111#2#11+111#111#..",
".#####......22&=&22+1111111111+2+11#111+111#..",
"2222222222222222222#1111111111#2########+###..",
"............2##/##2############2.......#111#..",
"............2#111#22222222222222.......#####..",
"............2#111#.............2..............",
"............2#####.............2..............",
"............2..................2..............",
"............2..................2..............",
};

PALETTE_MAP small_town_map[] = {
	{'1', M_STONEFLOOR},
	{'2', M_ROAD},
};


char * small_village_view[] = {
"                                                       ",
"                        =======                        ",
"          @@@@@@@  @@@@@@      ==                      ",
"        ..@;;;;;@..@....@........=                     ",
"        ..@@@+@@@..@+@@@@.........=                    ",
"        .....1......1.............=                    ",
"   1111111111111111111111111111...=                    ",
"       ...........1...####....11..=                    ",
"       ...........1##+#;;#.....111211111               ",
"       ...........1#;;;;;#........=     11111111111111 ",
"       ...........1#######.......==                    ",
"     .............1.............===                    ",
"     ...@@@@@@@@@@1....... ==========                  ",
"     ...@......+.+111111==============                 ",
"     ...@.....@@@@.====2===============                ",
"     ...@@@@@@@. ======2==================             ",
"     ........==========2====================           ",
"        ===============2======================         ",
"        ===============2=======================        ",
"      ================222======================        ",
"        ==============222====================          ",
"     =================222==================            ",
"      ==================================               ",
"       ===================================             ",
"        ================================               ",
"         ============================                  ",
"        ==============================                 ",
"          ==========================                   ",
"             =======================                   ",
"                   ============                        ",
"                                                       ",
};


PALETTE_MAP small_village_map[] = {
	{'@', M_WOODWALL},
	{'1', M_ROAD},
	{'3', M_SAND},
	{'2', M_BRIDGE},
};


char * city_view[] = {
"  =   ###.;;;;;;;;;;;;;;;;;;;;;;.MM  ##2##.....................##2##",
"  ==  #;#.;#####;...WWWWWWWWW..;;.MM #;;;#......#########......#;;;#",
"  ==  #;#.;#;;;#;...W;;;;;;;W...;..MM2;;;########;;W3W;;########;;;2",
"  ==  #;#.;#;;;+;...WW;;;;;WW...;...M#;;;+;;;;#;#;;;3;;;#;;;;#;+;;;#",
"  ==  #;#.;#####;....W;;;;;W...;;...M###+####;+;##;;3;;##;;;;#;#+###",
"  ==  #;#.;#;;;#;###.WW;;;WW.;;;....M..#;#;;#;#;;#;;3;;#;;;;;#;#;#..",
"  ==  #;#.;#;;;#;#;##.WW/WW.;;......M..#;#;;#;#####;3;##+#####;#;#..",
"   =  #;#.;#;;;+;#;;##.;;;;;;WWWWWWWM..#;#;;+;;;;;+;3;+;;;;;;;;#;#..",
"   =  #;#.;#;;;#;#;;;#.;;;...W;W===WMM.#;#;;#;######+#######+###;#..",
"   = ##+##;#####;##+##;;;;;..W;W=XXWWM.#;#;;#;#.....;.....#;;;;#;#..",
";;;1;#;;;#;;;;;;;;;;;;;=;=;;;W;W=X33WM.#;####;#....;;;....#;;;;#;#..",
";;;1;+;;;+;;;;;;;;;;;;;;;;;;;/333333WM.#;#;;+;+;;;;;=;;;;;+;;;;#;#..",
";;;1;#;;;#;;;;;;;;;;;;;=;=;;;W;W=X33WM.#;#;;###....;;;....###+##;#..",
"   = ##+##;#####;##+##;;;;;..W;W=XXWWM.#;#;;#;#.....;.....#;;;;#;#..",
"   =  #;#.;#;;;#;#;;;##;;;...W;W===WMM.#;#;;+;#..###+###..#;;;;#;#..",
"  ==  #;#.;#;;;#;#;;;;#;;;...WWWWWWWM###+#########;;3;;#########+###",
"  ==  #;#.;#;;;+;######;;;..........M#;;;+;;;;;;;+;;3;;+;;;;;;;+;;;#",
"  ==  #;#.;#####;#;;;;#;;;#########MM2;;;#########;;3;;#########;;;2",
"  ==  #;#.;#;;;#;#;;;;#;;;#;;#;;;;#M.#;;;#.......###+###T.T.T.T#;;;#",
"   == #;#.;#;;;#;#;;;;/;;;+;;#;;;;#MM##2##........M;;;MMMMM.TT.##2##",
"   == #;#.;#;;;#;#;;;;#;;;######+##.MMMMMMMMMMMMMMM;;;####MMMM.TT.T.",
"   == #;#.;##+##;######;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;+;;####MMM.T.T",
"   ==##+##;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;.#;;+;;#..MMMMM",
"   =##;;;##...........##+##...........................#;;#;;#.......",
"  ==#;;;;;#############;;;#############################+############",
"  ==#;;;;;+;;;;;;;;;;;+;;;+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;#",
"  ==#;;;;;#############;;;##########################################",
"   =##;;;##    =======##+##               ====     ========         ",
"   ==#####  ===========111====         =============================",
"    ==========         ;;;=================    ==           =====   "
};

PALETTE_MAP city_map[] = {
	{'1', M_BRIDGE},
	{'2', M_WINDOW},
	{'3', M_GOLDENFLOOR},
	{'M', M_MOUNTAIN},
	{'W', M_MARBLEWALL},
	{'X', M_GOLDENFENCE},
};




char * dwarf_city_view[] = {
"################################################################################",
"###############,,,,,,,############,,,,,,,###################,,,,,,,,,,,,,,,,,,,#",
"##,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"###,,,#########################,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"##,,,,#,#,#,#,#,#,#,#,#,#,#,#,#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"###,,,#+#+#+#+#+#+#+#+#+#+#+#+#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"####,,;;;;;;;;;;;;;;;;;;;;;;;;;;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"##,,,,#+#+#+#+#+#+#+#+#+#+#+#+#;,,,,,,,,,;;;;;;;;;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"###,,,#,#,#,#,#,#,#,#,#,#,#,#,#;,,,,,,,,,;;;;;;;;;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"##,,,,#########################;,,,,,,,,,;;==:==;;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"###,,,,,,,,,,,,,,,,,,,,########;,,,,,,,,,;;==:==;;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"######,,,,,,,,,,,,,,,,,#,,,,,,#;,,,,,,,,,;;::;::;;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#",
"#######,,,,,,,,,,,,,,,,#,,,,,,#;,,,,,,,,,;;==:==;;,,,,,,,,######################",
"##########,,,,,,,,,,,,,#,,,,,,+;;;;;;;;;;;;==:==;;;;;;;;;;+,,,,,,,,,,,,,,,,,,,##",
"###,,##,,##,,,,,,,,,,,,#,,,,,,#;,,,,,,,,,;;;;;;;;;,,,,,,,,####+#################",
"##,,,,,,,,#,,,,,,,,,,,,########;,,,,,,,,,;;;;;;;;;,,,,,,,,#,,,,,,,,#,,,#########",
"#,,,,,,,,,+;;;;;;;;;;;;;;;;;;;;;,,,,,,,,,,,,,,,,,,,,,,,,,,#,,,,,,,,+,,,++,,,,,##",
"#,,,,,,,,,#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#,,,,,,,,#,,,##,,,,,##",
"##,,,,,,,,#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,######################",
"################################################################################"
};


PALETTE_MAP dwarf_city_map[] = {
	{';', M_OBSIDIANFLOOR},
	{':', M_BRIDGE},
};


char * wizard_tower_view[] = {
"XXXXXXXXXXXXXXXXXXXXX",
"X...................X",
"X......#######......X",
"X.....##22#22##.....X",
"X....##222#222##....X",
"X....#22#####22#....X",
"X....##2/222/2##....X",
"X.....###222###.....X",
"X......###/###......X",
"X........111........X",
"XXXXXXXXX111XXXXXXXXX"
};

PALETTE_MAP wizard_tower_map[] = {
	{'#', M_MARBLEWALL},
	{'1', M_ROAD},
	{'2', M_GOLDENFLOOR},
};

char * wizard_tower_top_view[] = {
"XXXXXXXXXXXXXXXXXXXXX",
"X......##333##......X",
"X.....#3222223#.....X",
"X....##2222222##....X",
"X...#####222#####...X",
"X...3222#222#2223...X",
"X...##22/222/22##...X",
"X....3#2#222#2#3....X",
"X.....###222###.....X",
"X......##333##......X",
"XXXXXXXXX111XXXXXXXXX"
};

PALETTE_MAP wizard_tower_top_map[] = {
	{'#', M_MARBLEWALL},
	{'1', M_ROAD},
	{'2', M_GOLDENFLOOR},
	{'3', M_WINDOW}
};

char * rat_cellar_view[] = {
"################################################################################",
"#,,,,,,,,,,,,######################################################,,,,,,,,,,,,#",
"#,,,,,,,,,,,,+,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,+,,,,,,,,,,,,#",
"#,,,,,,,,,,,,###########################,##########################,,,,,,,,,,,,#",
"#,,,,,,,,,,,,###########################,###,,,,###################,,,,,,,,,,,,#",
"#,,,,,,,,,,,,###########################,###,,,,######,############,,,,,,,,,,,,#",
"######################,,,,,,,,##########,###,,,,######,#########################",
"########,,,,##########,######,##########,####+########+#########################",
"########,,,,+,,,,,,,##,#,,,,#,##########,####,,,,,,,,,,,,,,,,,,,,,+,,###########",
"########,,,,#######,##,#,##,#,##########,####,#####################,,###########",
"########,,,,#######,##,#,##,#,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,##,,###########",
"###################,##,#,##,############,#######################,###############",
"###################,##,,,##,############,#######################,###############",
"###################,#######,,,,,,,,,,###,#########,,############,###############",
"#,,,,,,,,,,,,######,################,###,#########,,+,,,,,,,,,,,,###,,,,,,,,,,,#",
"#,,,,,,,,,,,,######,,,,,,,,,,,,,,,,,,###,#########,,################,,,,,,,,,,,#",
"#,,,,,,,,,,,,###########################,###########################,,,,,,,,,,,#",
"#,,,,,,,,,,,,+,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,+,,,,,,,,,,,#",
"#,,,,,,,,,,,,#######################################################,,,,,,,,,,,#",
"################################################################################"
};

PALETTE_MAP rat_cellar_map[] = {
	{'#', M_STONEWALL}
};


char * tower_ruins_view[] = {
"....X..XX.XX..XX....X",
"X....................",
"........##.##.......X",
"........;..;;.#......",
".....#;;;;;...;#.....",
".....#;...;;;;;#.....",
"X......#;;..;........",
"........../##.......X",
"X....................",
"X...................X",
"....XX..X...X.....X.."
};

PALETTE_MAP tower_ruins_map[] = {
	{'#', M_BLACKMARBLEWALL},
};


char * ahkulan_castle_view[] = {
"################################################################################",
"##################==############################################################",
"##,,,,,,,,#####,,,==############################################################",
"##,,,,,,,,,,,,,,,==#############################################################",
"#,,,,,,,,,,,,,,,,==#############################################################",
"#,,,,,,,,,,,,,,,,==#############################################################",
"#,,,,,,,,,,,,,,,==#####################;;;;;;;##################################",
"#,,,,,,,,,,,,,,,==#;;##################;;;;;;;;#################################",
"#,,,,,,,,,,,,,,,==#;;##################;;;;;;;;;;;;#############################",
"#,,,;;;;;;;;;;;;;;+;;+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;#############################",
"#,,,,,,,,,,,,,,,==#;;##################;;;;;;;;;;;;#############################",
"#,,,,,,,,,,,,,,,==#;;##################;;;;;;;;#################################",
"#,,,,,,,,,,,,,,,==#####################;;;;;;;##################################",
"#,,,,,,,,,,,,,,,,==#############################################################",
"#,,,,,,,,,,,,,,,,==#############################################################",
"#,,,,,,,,,,,,,,,,==#############################################################",
"#,,,,,,,#,,,,,,,,,==############################################################",
"##,,,,####,,###,,,==############################################################",
"################################################################################",
"################################################################################"
};

PALETTE_MAP ahkulan_castle_map[] = {
	{';', M_OBSIDIANFLOOR},
};


char * dwarven_treasure_view[] = {
"#####################",
"#####,,,,,,,,,,,#####",
"##########^##########",
"#,,,######+######,,,#",
"#,,,,###,,,,,###,,,,#",
"#,,,,^+,,,,,,,+^,,,,#",
"#,,,,###,,,,,###,,,,#",
"#,,,######+#####,,,,#",
"##########^##########",
"#####,,,,,,,,,,,#####",
"#####################"
};

PALETTE_MAP dwarven_treasure_map[] = {
	{'#', M_STONEWALL}
};


char * undead_tomb1_view[] = {
"################################################################################",
"################################################################################",
"################################################################################",
"###################################################,,,,,,,,,####################",
"###################################################,,,,,,,,,+,,,,,,,,,,,########",
"#############################,,,,,,################,,,,,,,,,###########,########",
"#########################,,,,,,,,,,####################+###############,########",
"#######################,,,,,,,,,,,,##########,,,,,,####,####,,,,,,#####,########",
"###,,,,,##############,,,,,,,,,,,,,#########,,,,,,,,###,###,,,,,,,,####,########",
"###,,,,,##############,,,,,,,,,,,,,########,,######,,##+##,,######,,###+########",
"###,,,,,+,,,,,,,,,,,,+,,,,,,,,,,,,,+,,,,,,+,########,,+,+,,########,+,+,########",
"###,,,,,##############,,,,,,,,,,,,,########,,######,,##+##,,######,,###+########",
"###,,,,,##############,,,,,,,,,,,,,#########,,,,,,,,###,###,,,,,,,,####,########",
"#######################,,,,,,,,,,,,##########,,,,,,####,####,,,,,,#####,########",
"#########################,,,,,,,,,,####################+###############,########",
"#############################,,,,,,################,,,,,,,,,###########,########",
"###################################################,,,,,,,,,+,,,,,,,,,,,########",
"###################################################,,,,,,,,,####################",
"################################################################################",
"################################################################################"
};

PALETTE_MAP undead_tomb1_map[] = {
	{'#', M_STONEWALL}
};


char * undead_tomb0_view[] = {
"              ########",
"    ###########,,,,,,#",
"    #,,,,,,,,,,,,,,,##",
"#######,,,,,,,,,,##,# ",
"+,,+,,+,,,,,,,,,,#,,# ",
"#######,,,,,,,,,,##,# ",
"    #,,,,,,,,,,,,,,,##",
"    ###########,,,,,,#",
"              ########"
};

PALETTE_MAP undead_tomb0_map[] = {
	{'.', M_CAVEFLOOR},
	{'#', M_STONEWALL}
};



char * kings_treasure_view[] = {
"#####################",
"#######,,,,,,########",
"#####,,,,,^,,,,######",
"##,^,,,,,,,,,,,,,,,##",
"##^^,,^,,,^,,,^,,,,##",
"####,,,,,,,,,,,,,####",
"#######,,,,,,,#######",
"##########^##########",
"##########+##########",
"#########,,,#########",
"#####################"
};

PALETTE_MAP kings_treasure_map[] = {
	{'#', M_STONEWALL}
};


char * extinct_volcano_view[] = {
"      :    ,,,  ",
"  :: :;:  ,:::, ",
"    :;^;:,  ::, ",
"   :;^:^;:,,,,  ",
" ,,,:;^::  :::::",
"   ,,:;:::::    ",
"      :,,,      ",
};


PALETTE_MAP extinct_volcano_map[] = {

	{',', M_HILL},
	{':', M_LOWMOUNTAIN},
	{';', M_MOUNTAIN},
	{'^', M_HIGHMOUNTAIN},
};




/////////// RANDOM ROOMS /////////////
/*
char * random_room_1[] = {
"#####################",
"#,,,,,,,,,,,,,,,,,,,#",
"#,#################,#",
"#,#################,#",
"#,#################,#",
"#,#################,#",
"#,#################,#",
"#,#################,#",
"#,#################,#",
"#,,,,,,,,,,,,,,,,,,,#",
"#####################"
};

PALETTE_MAP kings_treasure_map[] = {
	{'#', M_STONEWALL}
};
*/



char ** XLocation::Resolve(PALETTE pal, int * size, PALETTE_MAP ** pm, int * pm_size)
{
	switch (pal)
	{
		case PAL_SMALL_TOWN : 
			*size = ARRAY_SIZE(small_town_view);
			*pm = small_town_map;
			*pm_size = ARRAY_SIZE(small_town_map);
			return small_town_view;
			break;

		case PAL_SMALL_VILLAGE : 
			*size = ARRAY_SIZE(small_village_view);
			*pm = small_village_map;
			*pm_size = ARRAY_SIZE(small_village_map);
			return small_village_view;
			break;

		case PAL_CITY : 
			*size = ARRAY_SIZE(city_view);
			*pm = city_map;
			*pm_size = ARRAY_SIZE(city_map);
			return city_view;
			break;

		case PAL_DWARF_CITY : 
			*size = ARRAY_SIZE(dwarf_city_view);
			*pm = dwarf_city_map;
			*pm_size = ARRAY_SIZE(dwarf_city_map);
			return dwarf_city_view;
			break;

		case PAL_WIZARD_TOWER : 
			*size = ARRAY_SIZE(wizard_tower_view);
			*pm = wizard_tower_map;
			*pm_size = ARRAY_SIZE(wizard_tower_map);
			return wizard_tower_view;
			break;

		case PAL_WIZTOWER_TOP :
			*size = ARRAY_SIZE(wizard_tower_top_view);
			*pm = wizard_tower_top_map;
			*pm_size = ARRAY_SIZE(wizard_tower_top_map);
			return wizard_tower_top_view;
			break;

		case PAL_AHKULAN_CASTLE : 
			*size = ARRAY_SIZE(ahkulan_castle_view);
			*pm = ahkulan_castle_map;
			*pm_size = ARRAY_SIZE(ahkulan_castle_map);
			return ahkulan_castle_view;
			break;

		case PAL_RAT_CELLAR : 
			*size = ARRAY_SIZE(rat_cellar_view);
			*pm = rat_cellar_map;
			*pm_size = ARRAY_SIZE(rat_cellar_map);
			return rat_cellar_view;
			break;

		case PAL_TOWER_RUINS : 
			*size = ARRAY_SIZE(tower_ruins_view);
			*pm = tower_ruins_map;
			*pm_size = ARRAY_SIZE(tower_ruins_map);
			return tower_ruins_view;
			break;

		case PAL_DWARF_TREASURE : 
			*size = ARRAY_SIZE(dwarven_treasure_view);
			*pm = dwarven_treasure_map;
			*pm_size = ARRAY_SIZE(dwarven_treasure_map);
			return dwarven_treasure_view;
			break;

		case PAL_KINGS_TREASURE : 
			*size = ARRAY_SIZE(kings_treasure_view);
			*pm = kings_treasure_map;
			*pm_size = ARRAY_SIZE(kings_treasure_map);
			return kings_treasure_view;
			break;

		case PAL_UNDEAD_TOMB0 : 
			*size = ARRAY_SIZE(undead_tomb0_view);
			*pm = undead_tomb0_map;
			*pm_size = ARRAY_SIZE(undead_tomb0_map);
			return undead_tomb0_view;
			break;

		case PAL_UNDEAD_TOMB1 : 
			*size = ARRAY_SIZE(undead_tomb1_view);
			*pm = undead_tomb1_map;
			*pm_size = ARRAY_SIZE(undead_tomb1_map);
			return undead_tomb1_view;
			break;

		case PAL_EXTINCT_VOLCANO : 
			*size = ARRAY_SIZE(extinct_volcano_view);
			*pm = extinct_volcano_map;
			*pm_size = ARRAY_SIZE(extinct_volcano_map);
			return extinct_volcano_view;
			break;



	}
	return NULL;
}

STDMAP best_fit_terrain_table[] = 
{
	M_GREENGRAS, 
	M_SAND, 
	M_PATH,
	M_ROAD,
	M_CAVEFLOOR, 
	M_STONEFLOOR, 
	M_OBSIDIANFLOOR,
	M_GOLDENFLOOR,
};

void XLocation::PutPalette(int x, int y, PALETTE pal, XLocation * l)
{

	int size;
	int mp_size;
	PALETTE_MAP * pm;
	char ** palette = Resolve(pal, &size, &pm, &mp_size);
	assert(palette);
	int len = strlen(palette[0]);

	XQList<XPoint> points_to_resolve;

	for (int i = 0; i < size; i++)
	{
		for (int j = 0; j < len; j++)
		{
			bool found_it = false;
			for(int k = 0; k < mp_size; k++)
			{
				if (pm[k].this_view == palette[i][j])
				{
					l->map->SetXY(x + j, y + i, pm[k].real_view);
					found_it = true;
					break;
				}
			}

			if (!found_it)
			{
				switch (palette[i][j])
				{
				case '+':
					new XDoor(x + j, y + i, 0, l);
					points_to_resolve.push_back(XPoint(x + j, y + i));
					break;

				case '/':
					new XDoor(x + j, y + i, 1, l);
					points_to_resolve.push_back(XPoint(x + j, y + i));
					break;

				case '^':
					new XTrap(x + j, y + i, l);
					points_to_resolve.push_back(XPoint(x + j, y + i));
					break;

				case '.':
					l->map->SetXY(x + j, y + i, M_GREENGRAS);
					break;
				
				case ',':
					l->map->SetXY(x + j, y + i, M_CAVEFLOOR);
					break;
				
				case ';':
					l->map->SetXY(x + j, y + i, M_STONEFLOOR);
					break;

				case '#':
					l->map->SetXY(x + j, y + i, M_STONEWALL);
					break;

				case '=':
					l->map->SetXY(x + j, y + i, M_WATER);
					break;

				case '&':
					l->map->SetXY(x + j, y + i, M_GREENTREE);
					break;

				case 'X':
					l->map->SetXY(x + j, y + i, M_FENCE);
					break;

				case ' ':
					break;

				default:
					points_to_resolve.push_back(XPoint(x + j, y + i));
					break;
				}

			}
		}
	}

	for (XQList<XPoint>::iterator it = points_to_resolve.begin(); it != points_to_resolve.end(); it++)
	{
		XPoint pt = *it;
		int best_fit_index = 0;
		for (int q = -1; q <= 1; q++)
		{
			for (int w = -1; w <= 1; w++)
			{
				if (q != 0 || w != 0)
				{
					STDMAP tm = l->map->GetXY(pt.x + q, pt.y + w);
					for (int i = 0; i < ARRAY_SIZE(best_fit_terrain_table); i++)
					{
						if (best_fit_terrain_table[i] == tm && best_fit_index < i)
							best_fit_index = i;
					}
				}
			}
		}
		l->map->SetXY(pt.x, pt.y, best_fit_terrain_table[best_fit_index]);
	}
}


