/*
This file is part of "Avanor, the Land of Mystery" roguelike game
Home page: http://www.avanor.com/
Copyright (C) 2000-2003 Vadim Gaidukevich

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __ITEMDEF_H
#define __ITEMDEF_H

enum BRAND_TYPE {
	BR_NONE				= 0x0000,
	BR_FIRE				= 0x0001, //weapon dmg + eq fire dmg
	BR_HELLFIRE			= 0x0002, //weapon dmg + eq x 3 fire dmg
	BR_COLD				= 0x0004,
	BR_ULTIMATECOLD		= 0x0008,
	BR_LIGHTNING		= 0x0010,
	BR_ELEMENTAL_MASK	= 0x001F, //mask for 'elemental'
	
	BR_ACID				= 0x0020,
	BR_POISON			= 0x0040,
	BR_DEATH			= 0x0080, //triple chance to score critical hit
	BR_BLACK_MASK		= 0x00E0, //mask for black

	BR_UNDEADSLAYER		= 0x0100, //3x vs undead
	BR_HUMANOIDSLAYER	= 0x0200,
	BR_ANIMALSLAYER		= 0x0400,
	BR_DRAGONSLAYER		= 0x0800,
	BR_GIANTSLAYER		= 0x1000,
	BR_ORCSLAYER		= 0x2000,
	BR_TROLLSLAYER		= 0x4000,
	BR_DEMONSLAYER		= 0x8000,
	BR_SLAYER_MASK		= 0xFF00,
	
	BR_HOLYSLAYER		= BR_UNDEADSLAYER | BR_DEMONSLAYER,
	BR_EVILSLAYER		= BR_DRAGONSLAYER | BR_GIANTSLAYER | BR_ORCSLAYER | BR_TROLLSLAYER,
	BR_ANY				= 0xFFFF
};

enum CAN_FLAG {
CAN_UNKNOWN		= 0x0000,

CAN_BROKE		= 0x0001,
CAN_RUST		= 0x0002,
CAN_CORD		= 0x0004,
CAN_DUST		= 0x0008,
CAN_BURN		= 0x0010,
CAN_MELT		= 0x0020,
CAN_FROST		= 0x0040,
CAN_LEATHERLIKE	= CAN_BURN,
CAN_GOODMETALLIKE	= CAN_BROKE | CAN_MELT,
CAN_BLACKMETALLIKE	= CAN_GOODMETALLIKE | CAN_CORD | CAN_RUST,
CAN_WOODLIKE		= CAN_BROKE | CAN_BURN,
CAN_STONELIKE		= CAN_DUST,
CAN_POTIONLIKE		= CAN_FROST | CAN_DUST,
CAN_PAPERLIKE		= CAN_BURN,
CAN_FIRERESIST		= 0xffff ^ (CAN_BURN | CAN_MELT),
CAN_COLDRESIST		= 0xffff ^ (CAN_FROST),
CAN_ACIDRESIST		= 0xffff ^ (CAN_CORD),
CAN_RESISTNONE		= 0xffff,
CAN_NICEMETALLIKE	= 0,
CAN_RESISTALL		= 0
};

enum SPECIAL_PROPERTY
{
	SPP_NONE				= 0x0000,
	SPP_FASTDIGESTION		= 0x0001,
	SPP_SLOWDIGESTION		= 0x0002,
	SPP_INVISIBLE			= 0x0004,
	SPP_REGENERATION		= 0x0008
};

struct _ITEMPROP
{
	char * propname; //wooden, metall,
	int color;
	char * dv; //modifers....
	char * pv;
	char * hit;
	char * dice;
	char * z; //random z to dice;
	char * r; //range
	unsigned int iflag; //wich item can consist from this material
	int probability;
	int density;
	int value;
	CAN_FLAG mflag;
	char * resistance; //to monster
	SPECIAL_PROPERTY sp;
};

enum ITEM_SET {
ISET_CLOTH			= 0x00000001,
ISET_LEATHER		= 0x00000002,
ISET_STUDEDLEATHER	= 0x00000004,
ISET_STONE			= 0x00000008,
ISET_WOOD			= 0x00000010,
ISET_IRON			= 0x00000020,
ISET_BRONZE			= 0x00000040,
ISET_BRASS			= 0x00000080,
ISET_SILVER			= 0x00000100,
ISET_GOLD			= 0x00000200,
ISET_CRYSTAL		= 0x00000400,
ISET_STEEL			= 0x00000800,
ISET_OBSIDIAN		= 0x00001000,
ISET_MITHRIL		= 0x00002000,
ISET_ADAMANTIUM		= 0x00004000,

ISET_SOFT		= ISET_LEATHER | ISET_STUDEDLEATHER | ISET_CLOTH,
ISET_ALLLEATHER	= ISET_LEATHER | ISET_STUDEDLEATHER,
ISET_METAL		= ISET_IRON | ISET_BRONZE | ISET_BRASS | ISET_SILVER | ISET_GOLD,
ISET_METALSOFT	= ISET_METAL | ISET_SOFT,
ISET_HARDMETAL	= ISET_STEEL | ISET_MITHRIL | ISET_ADAMANTIUM,
ISET_ALLMETAL	= ISET_METAL | ISET_HARDMETAL | ISET_OBSIDIAN,
ISET_OBSIMETAL	= ISET_OBSIDIAN | ISET_HARDMETAL,
ISET_STONEFROM	= ISET_STONE | ISET_CRYSTAL | ISET_OBSIDIAN,
ISET_WOODEN		= ISET_WOOD, //wooden
ISET_SHIELD		= ISET_ALLLEATHER | ISET_ALLMETAL | ISET_WOODEN, //for shields
ISET_BOW		= ISET_WOODEN,
ISET_MISSILE	= ISET_ALLMETAL | ISET_WOODEN | ISET_STONE,
ISET_WOODSTONE	= ISET_STONEFROM | ISET_WOODEN,
ISET_WEAPON		= ISET_STONEFROM | ISET_HARDMETAL,
ISET_SIMPLEWEAPON = ISET_IRON | ISET_STEEL,
ISET_CROWNMETAL	= ISET_STEEL | ISET_SILVER | ISET_GOLD | ISET_MITHRIL,
ISET_BLACKMETAL	= ISET_STEEL | ISET_IRON,
ISET_OTHER		= 0x0,
ISET_ALL		= 0x00000fff | ISET_MITHRIL | ISET_ADAMANTIUM,
};



enum WSK_TYPE {WSK_UNKNOWN = -1,
WSK_UNARMED = 0,
WSK_DAGGER, //knife, daggers
WSK_SWORD, //all swords
WSK_CLUB, //all clubs, hummers
WSK_MACE, //all maces, flails
WSK_POLEARM, //all pole arm, spere, pike etc.
WSK_AXE, //all axes
WSK_STAVE, //staves, staffs
WSK_SHIELD, //shields
WSK_BOW,
WSK_CROSSBOW,
WSK_SLING,
WSK_THROW,
WSK_EOF,
WSK_OTHER
};


enum ITEM_TYPE {
IT_UNKNOWN = 0,

IT_HAT, IT_CAP, IT_HELMET,

IT_AMULET, IT_NECKLACE,

IT_RING,

IT_CLUB, IT_WARHAMMER,

IT_DAGGER, IT_KNIFE, IT_ORCISHDAGGER, IT_LONGDAGGER,

IT_SHORTSWORD, IT_LONGSWORD, IT_BROADSWORD, IT_RAPIER, IT_SCIMITAR, IT_KATANA, IT_WAKIZASHI,

IT_SMALLAXE, IT_WARAXE, IT_BATTLEAXE, IT_GREATAXE, IT_ORCISHAXE,

IT_MACE, IT_FLAIL,

IT_SHORTSPEAR, IT_LONGSPEAR, IT_PITCHFORK, IT_PIKE, IT_HALBERD,

IT_STAFF,

IT_SHORTBOW, IT_LONGBOW, IT_LIGHTCROSSBOW, IT_CROSSBOW, IT_HEAVYCROSSBOW, IT_SLING,

IT_GLOVES, IT_GAUNTLETS, IT_KNUCKLES,

IT_SMALLSHIELD, IT_MEDIUMSHIELD, IT_LARGESHIELD, IT_TOWERSHIELD,

IT_SANDALS, IT_LIGHTBOOTS, IT_SOFTBOOTS, IT_HARDBOOTS,

IT_CLOTHES, IT_DRESS, IT_ROBE, IT_LIGHTMAIL, IT_SCALEMAIL, IT_PLATEMAIL, IT_CHAINMAIL, IT_RINGMAIL,

IT_CLOAK, IT_SHADOWCLOAK, IT_CAPE, IT_LIGHTCLOAK, IT_FORESTBROTHERCLOAK,

IT_TORCH,
IT_SCROLL,
IT_BOOK,
IT_POTION,
IT_HERB,
IT_LARGERATION, IT_RATION, IT_SMALLRATION, IT_ELVISHWAYBREAD, IT_CORPSE, IT_BONE, IT_RATTAIL, IT_BATWING,

IT_ARROW, IT_QUARREL, IT_SLINGBULLET, IT_ROCK,

IT_COOKINGSET, IT_PICKAXE, IT_ANCIENTMACHINEPART, IT_EYEOFRAA, IT_ALCHEMYSET,

IT_CHEST,

IT_MONEY,
IT_GEM,
IT_OTHER,
IT_RANDOM,
IT_EOF
};

enum USE_ITEM_STATE
{
	UIS_BEGIN,
	UIS_CONTINUE,
	UIS_STOP,
};


#endif
